package cn.com.qmth.stmms.biz.utils;

import org.springframework.data.redis.core.StringRedisTemplate;

import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.common.redis.RedisKeyBuilder;

public class MarkRedisUtil {

    public static void addCurrentTask(StringRedisTemplate redisTemplate, Marker marker, int libraryId) {
        String key = getKey(marker);
        redisTemplate.opsForZSet().add(key, String.valueOf(libraryId), marker.getId());
    }

    public static int countCurrentTask(StringRedisTemplate redisTemplate, int examId, String subjectCode, int number) {
        String key = RedisKeyBuilder.getCurrentTaskKey(examId, subjectCode, number);
        Long count = redisTemplate.opsForZSet().size(key);
        return count != null ? count.intValue() : 0;
    }

    public static int countCurrentTask(StringRedisTemplate redisTemplate, Marker marker) {
        String key = getKey(marker);
        Long count = redisTemplate.opsForZSet().count(key, marker.getId() - 0.5, marker.getId() + 0.5);
        return count != null ? count.intValue() : 0;
    }

    public static void removeCurrentTask(StringRedisTemplate redisTemplate, int examId, String subjectCode, int number,
            int libraryId) {
        String key = RedisKeyBuilder.getCurrentTaskKey(examId, subjectCode, number);
        redisTemplate.opsForZSet().remove(key, String.valueOf(libraryId));
    }

    public static boolean isCurrentTask(StringRedisTemplate redisTemplate, int examId, String subjectCode, int number,
            int libraryId) {
        String key = RedisKeyBuilder.getCurrentTaskKey(examId, subjectCode, number);
        Double score = redisTemplate.opsForZSet().score(key, String.valueOf(libraryId));
        return score != null && score > 0;
    }

    public static void clearCurrentTask(StringRedisTemplate redisTemplate, Marker marker) {
        String key = getKey(marker);
        redisTemplate.opsForZSet().removeRangeByScore(key, marker.getId() - 0.5, marker.getId() + 0.5);
    }

    public static void clearCurrentTask(StringRedisTemplate redisTemplate, int examId, String subjectCode, int number) {
        String key = RedisKeyBuilder.getCurrentTaskKey(examId, subjectCode, number);
        redisTemplate.delete(key);
    }

    private static String getKey(Marker marker) {
        return RedisKeyBuilder.getCurrentTaskKey(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
    }
}
